@@ -18,11 +18,12 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: TimeManagement.cc,v 3.32 2008/04/23 07:36:01 siron Exp $
+// $Id: TimeManagement.cc,v 3.48 2009/04/21 13:54:02 siron Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "TimeManagement.hh"
+#include "NM_Classes.hh"
 #include <float.h>
 
 namespace certi {
@@ -32,7 +33,7 @@ namespace {
 
 PrettyDebug D("RTIA_TM", __FILE__);
 static PrettyDebug G("GENDOC",__FILE__) ;
-const double epsilon = 1.0e-6 ;
+const double epsilon2 = 1.0e-4 ;
 
 }
 
@@ -69,6 +70,7 @@ TimeManagement::advance(bool &msg_restant, TypeException &e)
 TimeManagement::TimeManagement(Communications *GC,
                                Queues *GQueues,
                                FederationManagement *GF,
+                               DeclarationManagement *GD,
                                ObjectManagement *GO,
                                OwnershipManagement *GP)
     : LBTS()
@@ -76,13 +78,14 @@ TimeManagement::TimeManagement(Communications *GC,
     comm = GC ;
     queues = GQueues ;
     fm = GF ;
+    dm = GD ;
     om = GO ;
     owm = GP ;
 
     lastNullMessageDate = 0.0 ;
 
     _avancee_en_cours = PAS_D_AVANCEE ;
-    _blocking_tick         = false ;
+    _tick_state = NO_TICK;
     _asynchronous_delivery = false ;
 
     _heure_courante = 0.0 ;
@@ -95,23 +98,23 @@ TimeManagement::TimeManagement(Communications *GC,
 //! Send a null message to RTIG containing Local Time + Lookahead.
 void TimeManagement::sendNullMessage(FederationTime heure_logique)
 {
-    NetworkMessage msg ;
+    NM_Message_Null msg ;
 
+    msg.setDate(heure_logique);
     heure_logique += _lookahead_courant ;
 
     if (heure_logique > lastNullMessageDate) {
-        msg.type = NetworkMessage::MESSAGE_NULL ;
         msg.federation = fm->_numero_federation ;
         msg.federate = fm->federate ;
-        msg.date = heure_logique ;
+        msg.setDate(heure_logique) ; // ? See 6 lines upper !
 
         comm->sendMessage(&msg);
         lastNullMessageDate = heure_logique ;
-        D.Out(pdDebug, "NULL message sent (Time = %f).", heure_logique) ;
+        D.Out(pdDebug, "NULL message sent (Time = %f).", heure_logique.getTime()) ;
     }
     else {
         D.Out(pdExcept, "NULL message not sent (Time = %f, Last = %f).",
-              heure_logique, lastNullMessageDate);
+              heure_logique.getTime(), lastNullMessageDate.getTime());
     }
 }
 
@@ -120,18 +123,16 @@ void TimeManagement::sendNullMessage(FederationTime heure_logique)
 bool
 TimeManagement::executeFederateService(NetworkMessage &msg)
 {
-  G.Out(pdGendoc,"enter TimeManagement::executeFederateService for type %d",msg.type);
-  D.Out(pdRequest, "Execute federate service: Type %d.", msg.type);
+  G.Out(pdGendoc,"enter TimeManagement::executeFederateService for type %d",msg.getType());
+  D.Out(pdRequest, "Execute federate service: Type %d.", msg.getType());
 
-  msg.trace("TimeManagement::executeFederateService ");
+    _tick_state = TICK_NEXT;  // indicate the callback was processed
 
-    _blocking_tick = false;  // indicate a callback was processed
-
-    switch (msg.type) {
+    switch (msg.getType()) {
 
       case NetworkMessage::FEDERATION_SYNCHRONIZED:
         try {
-            fm->federationSynchronized(msg.label);
+            fm->federationSynchronized(msg.getLabel().c_str());
         }
         catch (RTIinternalError &e) {
             cout << "RTIA:RTIinternalError in federationSynchronized." << endl ;
@@ -141,7 +142,7 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
 
       case NetworkMessage::SYNCHRONIZATION_POINT_REGISTRATION_SUCCEEDED:
         try {
-            fm->synchronizationPointRegistrationSucceeded(msg.label);
+            fm->synchronizationPointRegistrationSucceeded(msg.getLabel().c_str());
         }
         catch (RTIinternalError &e) {
             cout << "RTIA:RTIinternalError in synchronizationPointRegistration"
@@ -152,7 +153,7 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
 
       case NetworkMessage::ANNOUNCE_SYNCHRONIZATION_POINT:
         try {
-            fm->announceSynchronizationPoint(msg.label, msg.tag);
+            fm->announceSynchronizationPoint(msg.getLabel().c_str(), msg.getTag().c_str());
         }
         catch (RTIinternalError &e) {
             cout << "RTIA:RTIinternalError in announceSynchronizationPoint." << endl ;
@@ -164,8 +165,8 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
         try {
             om->discoverObject(msg.object,
                                msg.objectClass,
-                               msg.label,
-                               msg.date,
+                               msg.getLabel().c_str(),
+                               msg.getDate(),
                                msg.eventRetraction,
                                msg.exception);
 
@@ -178,15 +179,15 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
 
       case NetworkMessage::REFLECT_ATTRIBUTE_VALUES:
       {
-          ValueLengthPair *ValueArray = msg.getAttribValueArray();
+          std::vector<AttributeValue_t> ValueArray = msg.getAttribValueArray();
 
-          if (msg.getBoolean())
+          if (msg.isDated())
              om->reflectAttributeValues(msg.object,
                                         msg.handleArray,
                                         ValueArray,
                                         msg.handleArraySize,
-                                        msg.date,
-                                        msg.label,
+                                        msg.getDate(),
+                                        msg.getLabel().c_str(),
                                         msg.eventRetraction,
                                         msg.exception);
           else
@@ -194,14 +195,14 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
                                         msg.handleArray,
                                         ValueArray,
                                         msg.handleArraySize,
-                                        msg.label,
+                                        msg.getLabel().c_str(),
                                         msg.exception);
-          free(ValueArray);
+          ValueArray.empty();
           break ;
       }
 
       case NetworkMessage::PROVIDE_ATTRIBUTE_VALUE_UPDATE:
-      { 
+      {
       om->provideAttributeValueUpdate(msg.object,
                                       msg.handleArray,
                                       msg.handleArraySize,
@@ -212,15 +213,15 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
 
       case NetworkMessage::RECEIVE_INTERACTION:
       {
-          ParameterLengthPair *ValueArray = msg.getParamValueArray();
+          std::vector<ParameterValue_t> ValueArray = msg.getParamValueArray();
 
-          if (msg.getBoolean())
+          if (msg.isDated())
               om->receiveInteraction(msg.interactionClass,
                                      msg.handleArray,
                                      ValueArray,
                                      msg.handleArraySize,
-                                     msg.date,
-                                     msg.label,
+                                     msg.getDate(),
+                                     msg.getLabel().c_str(),
                                      msg.eventRetraction,
                                      msg.exception);
           else
@@ -228,26 +229,26 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
                                      msg.handleArray,
                                      ValueArray,
                                      msg.handleArraySize,
-                                     msg.label,
+                                     msg.getLabel().c_str(),
                                      msg.exception);
-          free(ValueArray);
+          ValueArray.empty();
 
           break ;
       }
 
       case NetworkMessage::REMOVE_OBJECT:
-          if (msg.getBoolean()) {
+          if (msg.isDated()) {
         	om->removeObject(msg.object,
-                		 msg.federate,
-				 msg.date,
-                         	 msg.label,
+                		     msg.federate,
+				             msg.getDate(),
+                         	 msg.getLabel().c_str(),
                          	 msg.eventRetraction,
                          	 msg.exception);
 	  }
 	  else {
         	om->removeObject(msg.object,
-                		 msg.federate,
-                         	 msg.label,
+                		     msg.federate,
+                         	 msg.getLabel().c_str(),
                          	 msg.exception);
 	  }
         break ;
@@ -270,68 +271,80 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
         break ;
 
       case NetworkMessage::REQUEST_ATTRIBUTE_OWNERSHIP_ASSUMPTION:
+          {
         owm->requestAttributeOwnershipAssumption(msg.object,
                                                  msg.handleArray,
                                                  msg.handleArraySize,
                                                  msg.federate,
-                                                 msg.label,
+                                                 const_cast<char*>(msg.getLabel().c_str()),
                                                  msg.exception);
         break ;
+           }
 
       case NetworkMessage::ATTRIBUTE_OWNERSHIP_UNAVAILABLE:
+        {
         owm->attributeOwnershipUnavailable(msg.object,
                                            msg.handleArray,
                                            msg.handleArraySize,
                                            msg.federate,
                                            msg.exception);
         break ;
+        }
 
       case NetworkMessage::ATTRIBUTE_OWNERSHIP_ACQUISITION_NOTIFICATION:
+        {
         owm->attributeOwnershipAcquisitionNotification(msg.object,
                                                        msg.handleArray,
                                                        msg.handleArraySize,
                                                        msg.federate,
                                                        msg.exception);
         break ;
+        }
 
       case NetworkMessage::ATTRIBUTE_OWNERSHIP_DIVESTITURE_NOTIFICATION:
+        {
         owm->attributeOwnershipDivestitureNotification(msg.object,
                                                        msg.handleArray,
                                                        msg.handleArraySize,
                                                        msg.exception);
         break ;
+        }
 
       case NetworkMessage::REQUEST_ATTRIBUTE_OWNERSHIP_RELEASE:
+        {
         owm->requestAttributeOwnershipRelease(msg.object,
                                               msg.handleArray,
                                               msg.handleArraySize,
-                                              msg.label,
+                                              const_cast<char*>(msg.getLabel().c_str()),
                                               msg.exception);
         break ;
+        }
 
       case NetworkMessage::CONFIRM_ATTRIBUTE_OWNERSHIP_ACQUISITION_CANCELLATION:
+        {
         owm->confirmAttributeOwnershipAcquisitionCancellation(msg.object,
                                                               msg.handleArray,
                                                               msg.handleArraySize,
                                                               msg.exception);
         break ;
+        }
 
       case NetworkMessage::INITIATE_FEDERATE_SAVE:
-        fm->initiateFederateSave(msg.label);
+        fm->initiateFederateSave(msg.getLabel().c_str());
         break ;
 
       case NetworkMessage::FEDERATION_SAVED:
       case NetworkMessage::FEDERATION_NOT_SAVED: {
-          bool status = (msg.type == NetworkMessage::FEDERATION_SAVED) ? true : false ;
+          bool status = (msg.getType() == NetworkMessage::FEDERATION_SAVED) ? true : false ;
           fm->federationSavedStatus(status);
       }
         break ;
 
       case NetworkMessage::REQUEST_FEDERATION_RESTORE_SUCCEEDED:
       case NetworkMessage::REQUEST_FEDERATION_RESTORE_FAILED: {
-          bool status = (msg.type == NetworkMessage::REQUEST_FEDERATION_RESTORE_SUCCEEDED)
+          bool status = (msg.getType() == NetworkMessage::REQUEST_FEDERATION_RESTORE_SUCCEEDED)
               ? true : false ;
-          fm->requestFederationRestoreStatus(status, msg.label, msg.tag);
+          fm->requestFederationRestoreStatus(status, msg.getLabel().c_str(), msg.getTag().c_str());
       }
         break ;
 
@@ -340,20 +353,30 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
         break ;
 
       case NetworkMessage::INITIATE_FEDERATE_RESTORE:
-        fm->initiateFederateRestore(msg.label, msg.federate);
+        fm->initiateFederateRestore(msg.getLabel().c_str(), msg.federate);
         break ;
 
       case NetworkMessage::FEDERATION_RESTORED:
       case NetworkMessage::FEDERATION_NOT_RESTORED: {
-          bool status = (msg.type == NetworkMessage::FEDERATION_RESTORED) ? true : false ;
+          bool status = (msg.getType() == NetworkMessage::FEDERATION_RESTORED) ? true : false ;
           fm->federationRestoredStatus(status);
       }
         break ;
+      case NetworkMessage::TIME_REGULATION_ENABLED:
+        this->timeRegulationEnabled(msg.getDate(), msg.exception);
+        break;
+      case NetworkMessage::TIME_CONSTRAINED_ENABLED:
+        this->timeConstrainedEnabled(msg.getDate(), msg.exception);
+        break;
+      case NetworkMessage::START_REGISTRATION_FOR_OBJECT_CLASS:
+	dm->startRegistrationForObjectClass(msg.objectClass, msg.exception);
+        break;
 
       default:
+	std::stringstream errorMsg;
         D.Out(pdExcept, "Unknown message type in executeFederateService.");
-        msg.display("ERROR");
-        throw RTIinternalError("Unknown message in executeFederateService.");
+	errorMsg << "Unknown message <" <<  msg.getName() << " in executeFederateService.";
+        throw RTIinternalError(errorMsg.str().c_str());
     }
     G.Out(pdGendoc,"exit  TimeManagement::executeFederateService");
     return true ;
@@ -481,7 +504,7 @@ TimeManagement::nextEventRequest(FederationTime heure_logique,
         _type_granted_state = AFTER_TAR_OR_NER ;  // will be
 
         if (_lookahead_courant == 0.0) {
-           _lookahead_courant = epsilon ;
+           _lookahead_courant == epsilon2 ;
            _type_granted_state = AFTER_TAR_OR_NER_WITH_ZERO_LK ;
         }
 
@@ -544,7 +567,7 @@ TimeManagement::requestMinNextEventTime()
     FederationTime dateTSO ;
     FederationTime dateMNET ;
     bool found ;
- 
+
     queues->nextTsoDate(found, dateTSO) ;
 
     if (!found)
@@ -552,7 +575,7 @@ TimeManagement::requestMinNextEventTime()
     else
         dateMNET = (_LBTS <= dateTSO ? _LBTS : dateTSO) ;
 
-   D.Out(pdRegister, "Minimum Next Event Time : %f.", dateMNET);
+   D.Out(pdRegister, "Minimum Next Event Time : %f.", dateMNET.getTime());
 
    return dateMNET ;
 }
@@ -566,8 +589,12 @@ TimeManagement::setLookahead(FederationTimeDelta lookahead, TypeException &e)
     // Verifications
 
     if (lookahead < 0.0)
-        e = e_InvalidFederationTimeDelta ;
+        e = e_InvalidLookahead ;
 
+    if (lookahead == epsilon2) {
+    	cout << "Bad value of lookahead due to a zero lookahead implementation trick" << endl;
+    	e = e_RTIinternalError ;
+    }
 
     if (e == e_NO_EXCEPTION) {
         _lookahead_courant = lookahead ;
@@ -577,7 +604,7 @@ TimeManagement::setLookahead(FederationTimeDelta lookahead, TypeException &e)
         if (_est_regulateur)
             sendNullMessage(_heure_courante);
 
-        D.Out(pdRegister, "New Lookahead : %f.", _lookahead_courant);
+        D.Out(pdRegister, "New Lookahead : %f.", _lookahead_courant.getTime());
     }
 }
 
@@ -585,7 +612,7 @@ TimeManagement::setLookahead(FederationTimeDelta lookahead, TypeException &e)
 void
 TimeManagement::setTimeConstrained(bool etat, TypeException &e)
 {
-    NetworkMessage msg ;
+    NM_Set_Time_Constrained msg ;
 
     e = e_NO_EXCEPTION ;
 
@@ -600,10 +627,13 @@ TimeManagement::setTimeConstrained(bool etat, TypeException &e)
     if (e == e_NO_EXCEPTION) {
         _est_contraint = etat ;
 
-        msg.type = NetworkMessage::SET_TIME_CONSTRAINED ;
         msg.federation = fm->_numero_federation ;
         msg.federate = fm->federate ;
-        msg.constrained = etat ;
+        if (etat) {
+        	msg.constrainedOn();
+        } else {
+        	msg.constrainedOff();
+        }
 
         comm->sendMessage(&msg);
 
@@ -616,10 +646,13 @@ TimeManagement::setTimeConstrained(bool etat, TypeException &e)
 }
 
 // ----------------------------------------------------------------------------
+// Rajouter le time et le lookahead
+// Modifier le lookahead courant
 void
-TimeManagement::setTimeRegulating(bool etat, TypeException &e)
+TimeManagement::setTimeRegulating(bool etat,FederationTime heure_logique,
+               FederationTimeDelta the_lookahead, TypeException &e)
 {
-    NetworkMessage msg ;
+    NM_Set_Time_Regulating msg ;
 
     e = e_NO_EXCEPTION ;
 
@@ -640,11 +673,23 @@ TimeManagement::setTimeRegulating(bool etat, TypeException &e)
     if (e == e_NO_EXCEPTION) {
         _est_regulateur = etat ;
 
-        msg.type = NetworkMessage::SET_TIME_REGULATING ;
         msg.federation = fm->_numero_federation ;
         msg.federate = fm->federate ;
-        msg.regulator = etat ;
-        msg.date = _heure_courante + _lookahead_courant ;
+        if (etat) {
+        	msg.regulatorOn();
+        	D.Out(pdDebug,
+        		  "REGULATOR ON");
+        } else {
+        	msg.regulatorOff();
+        	D.Out(pdDebug,
+        	      "REGULATOR OFF");
+        }
+    // Modifier lookahead courant
+    _lookahead_courant = the_lookahead;
+          D.Out(pdDebug,
+    	        "New lookahead = %f",_lookahead_courant.getTime());
+    // faudrait peut etre remplacer heure courante par le temps en parametre
+        msg.setDate(_heure_courante + _lookahead_courant);
 
         comm->sendMessage(&msg);
 
@@ -656,6 +701,28 @@ TimeManagement::setTimeRegulating(bool etat, TypeException &e)
     }
 }
 
+// ----------------------------------------------------------------------------
+void
+TimeManagement::timeRegulationEnabled(FederationTime theTime, TypeException &e) {
+    Message req;
+
+    D.Out(pdDebug,"Sending TIME_REGULATION_ENABLED to Federate");
+    req.type = Message::TIME_REGULATION_ENABLED;
+    req.setFederationTime(theTime);
+    comm->requestFederateService(&req);
+}
+
+// ----------------------------------------------------------------------------
+void
+TimeManagement::timeConstrainedEnabled(FederationTime theTime, TypeException &e) {
+    Message req;
+
+    D.Out(pdDebug,"Sending TIME_CONSTRAINED_ENABLED to Federate");
+    req.type = Message::TIME_CONSTRAINED_ENABLED;
+    req.setFederationTime(theTime);
+    comm->requestFederateService(&req);
+}
+
 // ----------------------------------------------------------------------------
 /*! Is the time stamp of a time advance request correct ?
 */
@@ -665,24 +732,24 @@ TimeManagement::testValidTime(FederationTime theTime)
    if (_avancee_en_cours == PAS_D_AVANCEE) {
       if (_type_granted_state == AFTER_TAR_OR_NER_WITH_ZERO_LK) {
          if (theTime <= _heure_courante)
-            return 0;
+            return false;
       }
       else {  // AFTER_TAR_OR_NER or AFTER_TARA_OR_NARA
-         if (theTime < _heure_courante + _lookahead_courant)
-            return 0;
+         if (theTime  < _heure_courante + _lookahead_courant)
+            return false;
       }
    }
    else {
       if (_type_granted_state == AFTER_TAR_OR_NER_WITH_ZERO_LK) {
          if (theTime <= date_avancee)
-            return 0;
+            return false;
       }
       else {  // AFTER_TAR_OR_NER or AFTER_TARA_OR_NARA
-         if (theTime < date_avancee + _lookahead_courant)
-            return 0;
+         if (theTime  < date_avancee + _lookahead_courant)
+            return false;
       }
    }
-   return 1;
+   return true;
 }
 
 // ----------------------------------------------------------------------------
@@ -703,13 +770,19 @@ TimeManagement::tick(TypeException &e)
     // 1st try, give a command message. (requestPause, etc.)
     msg = queues->giveCommandMessage(msg_donne, msg_restant);
 
-
     // 2nd try, give a FIFO message. (discoverObject, etc.)
-    if (!msg_donne)
-        if ( _asynchronous_delivery || (_avancee_en_cours != PAS_D_AVANCEE) || (! _est_contraint))
+    if (!msg_donne){
+        if ( _asynchronous_delivery || (_avancee_en_cours != PAS_D_AVANCEE) || (! _est_contraint)) {
+        	D.Out(pdDebug,"FIFO message to be delivered async_deliver=%d, _avancee=%d, constrained=%d",
+        	        		  _asynchronous_delivery,_avancee_en_cours,_est_contraint);
           // to exclude the case not asynchronous_delivery and
           // not time advancing for a constrained federate
           msg = queues->giveFifoMessage(msg_donne, msg_restant);
+        } else {
+          D.Out(pdDebug,"FIFO message skipped async_deliver=%d, _avancee=%d, constrained=%d",
+        		  _asynchronous_delivery,_avancee_en_cours,_est_contraint);
+        }
+    }
 
     // If message exists, send it to federate.
     if (msg_donne) {
@@ -759,9 +832,9 @@ TimeManagement::timeAdvance(bool &msg_restant, TypeException &e)
     if (_est_contraint) {
         // give a TSO message.
         if (_LBTS == std::numeric_limits<double>::infinity())
-           D.Out(pdDebug, "Logical time : %f, LBTS : infini.", date_avancee);
+           D.Out(pdDebug, "Logical time : %f, LBTS : infini.", date_avancee.getTime());
         else
-           D.Out(pdDebug, "Logical time : %f, LBTS : %lf.", date_avancee, _LBTS);
+           D.Out(pdDebug, "Logical time : %f, LBTS : %lf.", date_avancee.getTime(), _LBTS.getTime());
         min = (_LBTS<date_avancee)?(_LBTS):(date_avancee);
         msg = queues->giveTsoMessage(min, msg_donne, msg_restant);
 
@@ -770,12 +843,12 @@ TimeManagement::timeAdvance(bool &msg_restant, TypeException &e)
             // if LBTS allows to give a timeAdvanceGrant.
             if (_LBTS == std::numeric_limits<double>::infinity())
                D.Out(pdDebug, "Logical time : %f, LBTS : infini, lookahead : %f.",
-                     date_avancee, _lookahead_courant);
+                     date_avancee.getTime(), _lookahead_courant.getTime());
             else
                D.Out(pdDebug, "Logical time : %15.12f, LBTS : %15.12f, lookahead : %f.",
-                     date_avancee, _LBTS, _lookahead_courant);
+                     date_avancee.getTime(), _LBTS.getTime(), _lookahead_courant.getTime());
 
-            if ((date_avancee < _LBTS) || 
+            if ((date_avancee < _LBTS) ||
                ((date_avancee == _LBTS) && (_avancee_en_cours == TARA))) {
                 // send a timeAdvanceGrant to federate.
                 timeAdvanceGrant(date_avancee, e);
@@ -816,12 +889,12 @@ TimeManagement::timeAdvanceGrant(FederationTime logical_time,
     req.setFederationTime(logical_time);
 
     D.Out(pdRegister, "timeAdvanceGrant sent to federate (time = %f).",
-          req.getFederationTime());
+          req.getFederationTime().getTime());
 
-    if (_lookahead_courant == epsilon)
-       _lookahead_courant = 0.0 ;
+    if (_lookahead_courant == epsilon2)
+        _lookahead_courant = 0.0 ;
 
-    _blocking_tick = false;  // indicate a callback was processed
+    _tick_state = TICK_NEXT;  // indicate the callback was processed
 
     comm->requestFederateService(&req);
 
@@ -847,15 +920,19 @@ TimeManagement::timeAdvanceRequest(FederationTime logical_time,
     if (logical_time < _heure_courante)
         e = e_FederationTimeAlreadyPassed ;
 
-    if (logical_time < _heure_courante + _lookahead_courant)
+    if (logical_time < _heure_courante + _lookahead_courant) {
+
+    D.Out(pdDebug,"InvalidFederation time lkahead=%f, current=%f, requested=%f",
+    			_lookahead_courant.getTime(),_heure_courante.getTime(),logical_time.getTime());
        e = e_InvalidFederationTime ;
+    }
 
     if (e == e_NO_EXCEPTION) {
 
         _type_granted_state = AFTER_TAR_OR_NER ;  // will be
 
         if (_lookahead_courant == 0.0) {
-           _lookahead_courant = epsilon ;
+           _lookahead_courant = epsilon2 ;
            _type_granted_state = AFTER_TAR_OR_NER_WITH_ZERO_LK ;
         }
 
@@ -866,7 +943,7 @@ TimeManagement::timeAdvanceRequest(FederationTime logical_time,
         date_avancee = logical_time ;
 
         D.Out(pdTrace, "timeAdvanceRequest accepted (asked time=%f).",
-              date_avancee);
+              date_avancee.getTime());
     }
     else {
         D.Out(pdExcept, "timeAdvanceRequest refused (exception = %d).", e);
@@ -902,7 +979,7 @@ TimeManagement::timeAdvanceRequestAvailable(FederationTime logical_time,
         date_avancee = logical_time ;
 
         D.Out(pdTrace, "timeAdvanceRequestAvailable accepted (asked time=%f).",
-              date_avancee);
+              date_avancee.getTime());
     }
     else {
         D.Out(pdExcept, "timeAdvanceRequestAvailable refused (exception = %d).", e);
@@ -912,4 +989,4 @@ TimeManagement::timeAdvanceRequestAvailable(FederationTime logical_time,
 
 }} // namespaces
 
-// $Id: TimeManagement.cc,v 3.32 2008/04/23 07:36:01 siron Exp $
+// $Id: TimeManagement.cc,v 3.48 2009/04/21 13:54:02 siron Exp $
